@@ -22,6 +22,7 @@ class WiredConnectThread() -- Connection thread for wired
 #   Copyright (C) 2007 - 2009 Adam Blackburn
 #   Copyright (C) 2007 - 2009 Dan O'Reilly
 #   Copyright (C) 2007 - 2009 Byron Hillis
+#   Copyright (C)        2009 Andrew Psaltis
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License Version 2 as
@@ -52,6 +53,7 @@ from signal import SIGTERM
 import misc
 import wpath
 from backend import BackendManager
+from translations import _
 
 if __name__ == '__main__':
     wpath.chdir(__file__)
@@ -156,25 +158,34 @@ class Controller(object):
         self.driver = None
         self.iface = None
     
-    def get_debug(self): return self._debug
+    def get_debug(self):
+        """ Getter for debug property. """
+        return self._debug
     def set_debug(self, value):
+        """ Setter for debug property. """
         self._debug = value
         if self.iface:
             self.iface.SetDebugMode(value)
     debug = property(get_debug, set_debug)
     
     def set_dhcp_client(self, value):
+        """ Setter for dhcp_client property. """
         self._dhcp_client = value
         if self.iface:
             self.iface.DHCP_CLIENT = value
-    def get_dhcp_client(self): return self._dhcp_client
+    def get_dhcp_client(self):
+        """ Getter for dhcp_client property. """
+        return self._dhcp_client
     dhcp_client = property(get_dhcp_client, set_dhcp_client)
     
     def set_flush_tool(self, value):
+        """ Setter for flush_tool property. """
         self._flush_tool = value
         if self.iface:
             self.iface.flush_tool = value
-    def get_flush_tool(self): return self._flush_tool
+    def get_flush_tool(self):
+        """ Getter for flush_tool property. """
+        return self._flush_tool
     flush_tool = property(get_flush_tool, set_flush_tool)
     
     def LoadBackend(self, backend_name):
@@ -201,25 +212,35 @@ class Controller(object):
         """
         return self.iface.GetIP(ifconfig)
 
-    def Disconnect(self, *args, **kargs):
+    def Disconnect(self, nettype, name, mac):
         """ Disconnect from the network. """
         iface = self.iface
-        misc.ExecuteScripts(wpath.predisconnectscripts, self.debug)
+        # mac and name need to be strings
+        if mac in (None, ''):
+            mac = 'X'
+        if name in (None, ''):
+            name = 'X'
+        misc.ExecuteScripts(wpath.predisconnectscripts, self.debug,
+                           extra_parameters=(nettype, name, mac))
         if self.pre_disconnect_script:
             print 'Running pre-disconnect script'
             misc.ExecuteScript(expand_script_macros(self.pre_disconnect_script,
-                                                    'pre-disconnection', *args),
+                                                    'pre-disconnection',
+                                                    mac, name),
                                self.debug)
         iface.ReleaseDHCP()
         iface.SetAddress('0.0.0.0')
         iface.FlushRoutes()
+        iface.FlushDNS()
         iface.Down()
         iface.Up()
-        misc.ExecuteScripts(wpath.postdisconnectscripts, self.debug)
+        misc.ExecuteScripts(wpath.postdisconnectscripts, self.debug,
+                            extra_parameters=(nettype, name, mac))
         if self.post_disconnect_script:
             print 'Running post-disconnect script'
             misc.ExecuteScript(expand_script_macros(self.post_disconnect_script,
-                                                    'post-disconnection', *args),
+                                                    'post-disconnection',
+                                                   mac, name),
                                self.debug)
         
     def ReleaseDHCP(self):
@@ -281,7 +302,8 @@ class ConnectThread(threading.Thread):
 
     def __init__(self, network, interface_name, before_script, after_script, 
                  pre_disconnect_script, post_disconnect_script, gdns1,
-                 gdns2, gdns3, gdns_dom, gsearch_dom, iface, debug):
+                 gdns2, gdns3, gdns_dom, gsearch_dom, iface,
+                 debug):
         """ Initialise the required object variables and the thread.
 
         Keyword arguments:
@@ -316,28 +338,34 @@ class ConnectThread(threading.Thread):
         self.global_dns_3 = gdns3
         self.global_dns_dom = gdns_dom
         self.global_search_dom = gsearch_dom
-        
+
         self.iface = iface
 
-        self.connecting_message = None
+        self.connecting_status = None
         self.debug = debug
         
         self.SetStatus('interface_down')
-        
+
+    def _connect(self):
+        raise NotImplementedError
+
     def run(self):
-        self.connect_result = "Failed"
+        self.connect_result = "failed"
         try:
             self._connect()
         finally:
             self.is_connecting = False
         
     def set_should_die(self, val):
+        """ Setter for should_die property. """
         self.lock.acquire()
         try:
             self._should_die = val
         finally:
             self.lock.release()
-    def get_should_die(self): return self._should_die
+    def get_should_die(self):
+        """ Getter for should_die property. """
+        return self._should_die
     should_die = property(get_should_die, set_should_die)
 
     def SetStatus(self, status):
@@ -349,7 +377,7 @@ class ConnectThread(threading.Thread):
         """
         self.lock.acquire()
         try:
-            self.connecting_message = status
+            self.connecting_status = status
         finally:
             self.lock.release()
 
@@ -362,10 +390,10 @@ class ConnectThread(threading.Thread):
         """
         self.lock.acquire()
         try:
-            message = self.connecting_message
+            status = self.connecting_status
         finally:
             self.lock.release()
-        return message
+        return status
     
     @abortable
     def reset_ip_addresses(self, iface):
@@ -387,8 +415,10 @@ class ConnectThread(threading.Thread):
         iface.Down()
         
     @abortable
-    def run_global_scripts_if_needed(self, script_dir):
-        misc.ExecuteScripts(script_dir, verbose=self.debug)
+    def run_global_scripts_if_needed(self, script_dir, extra_parameters=()):
+        """ Run global scripts if needed. '"""
+        misc.ExecuteScripts(script_dir, verbose=self.debug,
+                            extra_parameters=extra_parameters)
 
     @abortable
     def run_script_if_needed(self, script, msg, bssid='wired', essid='wired'):
@@ -436,35 +466,32 @@ class ConnectThread(threading.Thread):
         else:
             # Run dhcp...
             self.SetStatus('running_dhcp')
-            print "Running DHCP"
-            dhcp_status = iface.StartDHCP()
+            if self.network.get('usedhcphostname') == None:
+                self.network['usedhcphostname'] = False
+            if self.network.get('dhcphostname') == None:
+                self.network['dhcphostname'] = os.uname()[1]
+            if self.network['usedhcphostname']:
+                hname = self.network['dhcphostname']
+                print "Running DHCP with hostname", hname
+            else:
+                hname = None
+                print "Running DHCP with NO hostname"
+            dhcp_status = iface.StartDHCP(hname)
             if dhcp_status in ['no_dhcp_offers', 'dhcp_failed']:
                 if self.connect_result != "aborted":
                     self.abort_connection(dhcp_status)
                 return
 
     @abortable
-    def verify_association(self, iface):
-        """ Verify that our association the AP is valid.
-        
-        Try to ping the gateway we have set to see if we're
-        really associated with it.  This is only done if
-        we're using a static IP.
-        
+    def flush_dns_addresses(self, iface):
+        """ Flush the added DNS address(es).
+
+        This is only useful when using resolvconf, since we effectively have no
+        foolproof way of removing added DNS addresses from a non-resolvconf
+        setup.
+
         """
-        if self.network.get('gateway'):
-            self.SetStatus('verifying_association')
-            print "Verifying AP association"
-            retcode = self.iface.VerifyAPAssociation(self.network['gateway'])
-            #TODO this should be in wnettools.py
-            if retcode:
-                print "Connection Failed: Failed to ping the access point!"
-                # Clean up before aborting.
-                iface.SetAddress('0.0.0.0')
-                iface.FlushRoutes()
-                if hasattr(iface, "StopWPA"):
-                    iface.StopWPA()
-                self.abort_connection("association_failed")
+        iface.FlushDNS()
 
     @abortable
     def set_dns_addresses(self, iface):
@@ -499,7 +526,7 @@ class ConnectThread(threading.Thread):
         """ Sets the thread status to aborted. """
         if self.abort_reason:
             reason = self.abort_reason
-        self.connecting_message = reason
+        self.connecting_status = reason
         self.is_aborted = True
         self.connect_result = reason
         self.is_connecting = False
@@ -520,12 +547,26 @@ class ConnectThread(threading.Thread):
         finally:
             self.lock.release()
         
+    @abortable
+    def stop_wpa(self, iface):
+        """ Stops wpa_supplicant. """
+        print 'Stopping wpa_supplicant'
+        iface.StopWPA()
+        
     @abortable
     def put_iface_up(self, iface):
         """ Bring up given interface. """
         print 'Putting interface up...'
         self.SetStatus('interface_up')
         iface.Up()
+        for x in range(0, 5):
+            time.sleep(2)
+            if iface.IsUp():
+                return
+            self.abort_if_needed()
+         
+        # If we get here, the interface never came up
+        print "WARNING: Timed out waiting for interface to come up"
 
 
 class Wireless(Controller):
@@ -537,24 +578,33 @@ class Wireless(Controller):
         self._wpa_driver = None
         self._wireless_interface = None
         self.wiface = None 
+        self.should_verify_ap = True
         
     def set_wireless_iface(self, value):
+        """ Setter for wireless_interface property. """
         self._wireless_interface = value
         if self.wiface:
             self.wiface.SetInterface(value)
-    def get_wireless_iface(self): return self._wireless_interface
+    def get_wireless_iface(self):
+        """ Getter for wireless_interface property. """
+        return self._wireless_interface
     wireless_interface = property(get_wireless_iface, set_wireless_iface) 
         
     def set_wpa_driver(self, value):
+        """ Setter for wpa_driver property. """
         self._wpa_driver = value
         if self.wiface:
             self.SetWPADriver(value)
-    def get_wpa_driver(self): return self._wpa_driver
+    def get_wpa_driver(self):
+        """ Getter for wpa_driver property. """
+        return self._wpa_driver
     wpa_driver = property(get_wpa_driver, set_wpa_driver)
     
     def set_iface(self, value):
+        """ Setter for iface property. """
         self.wiface = value
     def get_iface(self):
+        """ Getter for iface property. """
         return self.wiface
     iface = property(get_iface, set_iface)
         
@@ -566,8 +616,9 @@ class Wireless(Controller):
         
         """
         Controller.LoadBackend(self, backend)
-        if self._backend:
-            self.wiface = self._backend.WirelessInterface(self.wireless_interface,
+        backend = self._backend
+        if backend:
+            self.wiface = backend.WirelessInterface(self.wireless_interface,
                                                     self.debug, self.wpa_driver)
 
     def Scan(self, essid=None):
@@ -581,13 +632,15 @@ class Wireless(Controller):
 
         """
         def comp(x, y):
+            """ Custom sorting function. """
             if 'quality' in x:
                 key = 'quality'
             else:
                 key = 'strength'
             return cmp(x[key], y[key])
                 
-        if not self.wiface: return []
+        if not self.wiface:
+            return []
         wiface = self.wiface
 
         # Prepare the interface for scanning
@@ -599,6 +652,8 @@ class Wireless(Controller):
         if essid is not None:
             print 'Setting hidden essid' + essid
             wiface.SetEssid(essid)
+            # sleep for a bit; scanning to fast will result in nothing
+            time.sleep(1)
 
         aps = wiface.GetNetworks()
         aps.sort(cmp=comp, reverse=True)
@@ -612,14 +667,16 @@ class Wireless(Controller):
         network -- network to connect to
 
         """
-        if not self.wiface: return False
+        if not self.wiface:
+            return False
         
         self.connecting_thread = WirelessConnectThread(network,
             self.wireless_interface, self.wpa_driver, self.before_script,
             self.after_script, self.pre_disconnect_script,
             self.post_disconnect_script, self.global_dns_1,
             self.global_dns_2, self.global_dns_3, self.global_dns_dom,
-            self.global_search_dom, self.wiface, debug)
+            self.global_search_dom, self.wiface, self.should_verify_ap,
+            self.bitrate, self.allow_lower_bitrates, debug)
         self.connecting_thread.setDaemon(True)
         self.connecting_thread.start()
         return True
@@ -649,6 +706,8 @@ class Wireless(Controller):
         The name of the currently connected network.
 
         """
+        if self.connecting_thread and self.connecting_thread.is_connecting:
+            return self.connecting_thread.network['essid']
         return self.wiface.GetCurrentNetwork(iwconfig)
     
     def GetBSSID(self):
@@ -691,16 +750,25 @@ class Wireless(Controller):
         """
         return self.wiface.GetAvailableAuthMethods(iwlistauth)
 
+    def GetAvailableBitrates(self):
+        """ Get the available bitrates for the interface.
+
+        Returns:
+        The available bitrates of the interface as a string, or None if the
+        bitrates can't be found.
+        """
+        return self.wiface.GetAvailableBitrates()
+
     def GetIwconfig(self):
         """ Get the out of iwconfig. """
         return self.wiface.GetIwconfig()
     
-    def GetWpaSupplicantDrivers(self, drivers):
-        """ Returns all valid wpa_supplicant drivers in a list. """
-        return [driver for driver in drivers if 
-                BACKEND.IsValidWpaSuppDriver(driver)]
+    def GetWpaSupplicantDrivers(self):
+        """ Returns all valid wpa_supplicant drivers on the system. """
+        return BACKEND.GetWpaSupplicantDrivers()
     
     def StopWPA(self):
+        """ Stop wpa_supplicant. """
         return self.wiface.StopWPA()
 
     def CreateAdHocNetwork(self, essid, channel, ip, enctype, key,
@@ -758,6 +826,38 @@ class Wireless(Controller):
         """
         return self.wiface.GetKillSwitchStatus()
 
+    def SwitchRfKill(self):
+        """ Switches the rfkill on/off for wireless cards. """
+        types = ['wifi', 'wlan', 'wimax', 'wwan']
+        try:
+            if self.GetRfKillStatus():
+                action = 'unblock'
+            else:
+                action = 'block'
+            for t in types:
+                cmd = ['rfkill', action, t]
+                print "rfkill: %sing %s" % (action, t)
+                misc.Run(cmd)
+            return True
+        except Exception, e:
+            raise e
+            return False
+
+    def GetRfKillStatus(self):
+        """ Determines if rfkill switch is active or not.
+
+        Returns:
+        True if rfkill (soft-)switch is enabled.
+        """
+        cmd = 'rfkill list'
+        rfkill_out = misc.Run(cmd)
+        soft_blocks = filter(lambda x: x.startswith('Soft'),
+            rfkill_out.split('\t'))
+        for line in map(lambda x: x.strip(), soft_blocks):
+            if line.endswith('yes'):
+                return True
+        return False
+
     def Disconnect(self):
         """ Disconnect the given iface.
         
@@ -769,10 +869,10 @@ class Wireless(Controller):
             iwconfig = self.GetIwconfig()
         else:
             iwconfig = None
-        bssid = self.wiface.GetBSSID(iwconfig),
+        bssid = self.wiface.GetBSSID(iwconfig)
         essid = self.wiface.GetCurrentNetwork(iwconfig) 
 
-        Controller.Disconnect(self, bssid, essid)
+        Controller.Disconnect(self, 'wireless', essid, bssid)
         self.StopWPA()
     
     def SetWPADriver(self, driver):
@@ -790,8 +890,8 @@ class WirelessConnectThread(ConnectThread):
 
     def __init__(self, network, wireless, wpa_driver, before_script,
                  after_script, pre_disconnect_script, post_disconnect_script,
-                 gdns1, gdns2, gdns3, gdns_dom, gsearch_dom, wiface,
-                 debug=False):
+                 gdns1, gdns2, gdns3, gdns_dom, gsearch_dom, wiface, 
+                 should_verify_ap, bitrate, allow_lower_bitrates, debug=False):
         """ Initialise the thread with network information.
 
         Keyword arguments:
@@ -805,6 +905,8 @@ class WirelessConnectThread(ConnectThread):
         gdns1 -- global DNS server 1
         gdns2 -- global DNS server 2
         gdns3 -- global DNS server 3
+        bitrate -- chosen interface bitrate
+        allow_lower_bitrates -- whether to allow lower bitrates or not
 
         """
         ConnectThread.__init__(self, network, wireless, before_script, 
@@ -812,7 +914,9 @@ class WirelessConnectThread(ConnectThread):
                                post_disconnect_script, gdns1, gdns2,
                                gdns3, gdns_dom, gsearch_dom, wiface, debug)
         self.wpa_driver = wpa_driver
-
+        self.should_verify_ap = should_verify_ap
+        self.bitrate = bitrate
+        self.allow_lower_bitrates = allow_lower_bitrates
 
     def _connect(self):
         """ The main function of the connection thread.
@@ -832,7 +936,11 @@ class WirelessConnectThread(ConnectThread):
         self.is_connecting = True
         
         # Run pre-connection script.
-        self.run_global_scripts_if_needed(wpath.preconnectscripts)
+        self.run_global_scripts_if_needed(wpath.preconnectscripts,
+                                          extra_parameters=('wireless',
+                                                    self.network['essid'],
+                                                    self.network['bssid'])
+                                         )
         self.run_script_if_needed(self.before_script, 'pre-connection', 
                                   self.network['bssid'], self.network['essid'])
 
@@ -843,7 +951,9 @@ class WirelessConnectThread(ConnectThread):
         self.reset_ip_addresses(wiface)
         self.stop_wpa(wiface)
         self.flush_routes(wiface)
+        self.flush_dns_addresses(wiface)
         wiface.SetMode(self.network['mode'])
+        wiface.SetBitrate(self.bitrate, self.allow_lower_bitrates)
 
         # Put interface up.
         self.SetStatus('configuring_interface')
@@ -866,7 +976,8 @@ class WirelessConnectThread(ConnectThread):
         if self.network.get('enctype'):
             self.SetStatus('validating_authentication')
             if not wiface.ValidateAuthentication(time.time()):
-                if not self.connect_result:
+                print "connect result is %s" % self.connect_result
+                if not self.connect_result or self.connect_result == 'failed':
                     self.abort_connection('bad_pass')
 
         # Set up gateway, IP address, and DNS servers.
@@ -876,7 +987,11 @@ class WirelessConnectThread(ConnectThread):
         self.verify_association(wiface)
         
         # Run post-connection script.
-        self.run_global_scripts_if_needed(wpath.postconnectscripts)
+        self.run_global_scripts_if_needed(wpath.postconnectscripts,
+                                          extra_parameters=('wireless',
+                                                    self.network['essid'],
+                                                    self.network['bssid'])
+                                         )
         self.run_script_if_needed(self.after_script, 'post-connection', 
                                   self.network['bssid'], self.network['essid'])
 
@@ -884,14 +999,40 @@ class WirelessConnectThread(ConnectThread):
         print 'Connecting thread exiting.'
         if self.debug:
             print "IP Address is: " + str(wiface.GetIP())
-        self.connect_result = "Success"
+        self.connect_result = "success"
         self.is_connecting = False
         
     @abortable
-    def stop_wpa(self, wiface):
-        """ Stops wpa_supplicant. """
-        print 'Stopping wpa_supplicant'
-        wiface.StopWPA()
+    def verify_association(self, iface):
+        """ Verify that our association the AP is valid.
+        
+        Try to ping the gateway we have set to see if we're
+        really associated with it.  This is only done if
+        we're using a static IP.
+        
+        """
+        if self.network.get('gateway') and self.should_verify_ap:
+            self.SetStatus('verifying_association')
+            print "Verifying AP association..."
+            for tries in range(1, 11):
+                print "Attempt %d of 10..." % tries
+                retcode = self.iface.VerifyAPAssociation(self.network['gateway'])
+                if retcode == 0: 
+                    print "Successfully associated."
+                    break
+                time.sleep(1)
+            #TODO this should be in wnettools.py
+            if retcode:
+                print "Connection Failed: Failed to ping the access point!"
+                # Clean up before aborting.
+                iface.SetAddress('0.0.0.0')
+                iface.FlushRoutes()
+                iface.FlushDNS()
+                if hasattr(iface, "StopWPA"):
+                    iface.StopWPA()
+                self.abort_connection('association_failed')
+        else:
+            print 'not verifying'
         
     @abortable
     def generate_psk_and_authenticate(self, wiface):
@@ -905,7 +1046,8 @@ class WirelessConnectThread(ConnectThread):
         # cards).
         if self.debug:
             print "enctype is %s" % self.network.get('enctype')
-        if self.network.get('key') and 'wpa' in str(self.network.get('enctype')):
+        if self.network.get('key') and \
+           'wpa' in str(self.network.get('enctype')):
             self.SetStatus('generating_psk')
             print 'Generating psk...'
             self.network['psk'] = wiface.GeneratePSK(self.network)
@@ -934,23 +1076,31 @@ class Wired(Controller):
         self.liface = None
         
     def set_link_detect(self, value):
+        """ Setter for link_detect property. """
         self._link_detect = value
         if self.liface:
             self.liface.link_detect = value
-    def get_link_detect(self): return self._link_detect
+    def get_link_detect(self):
+        """ Getter for link_detect property. """
+        return self._link_detect
     link_detect = property(get_link_detect, set_link_detect)
     
     
     def set_wired_iface(self, value):
+        """ Setter for wired_interface property. """
         self._wired_interface = value
         if self.liface:
             self.liface.SetInterface(value)            
-    def get_wired_iface(self): return self._wired_interface
+    def get_wired_iface(self):
+        """ Getter for wired_interface property. """
+        return self._wired_interface
     wired_interface = property(get_wired_iface, set_wired_iface)
     
     def set_iface(self, value):
+        """ Setter for iface property. """
         self.liface = value
     def get_iface(self):
+        """ Getter for iface property. """
         return self.liface
     iface = property(get_iface, set_iface)
     
@@ -977,7 +1127,8 @@ class Wired(Controller):
         network -- network to connect to
 
         """
-        if not self.liface: return False
+        if not self.liface:
+            return False
         self.connecting_thread = WiredConnectThread(network,
             self.wired_interface, self.before_script, self.after_script,
             self.pre_disconnect_script, self.post_disconnect_script,
@@ -989,7 +1140,12 @@ class Wired(Controller):
         return self.connecting_thread
     
     def Disconnect(self):
-        Controller.Disconnect(self, 'wired', 'wired')
+        Controller.Disconnect(self, 'wired', 'wired', 'wired')
+        self.StopWPA()
+    
+    def StopWPA(self):
+        """ Stop wpa_supplicant. """
+        self.liface.StopWPA()
     
     def DetectWiredInterface(self):
         """ Attempts to automatically detect a wired interface. """
@@ -1027,7 +1183,8 @@ class WiredConnectThread(ConnectThread):
         ConnectThread.__init__(self, network, wired, before_script, 
                                after_script, pre_disconnect_script,
                                post_disconnect_script, gdns1, gdns2,
-                               gdns3, gdns_dom, gsearch_dom, liface, debug)
+                               gdns3, gdns_dom, gsearch_dom, liface,
+                               debug)
 
     def _connect(self):
         """ The main function of the connection thread.
@@ -1048,27 +1205,39 @@ class WiredConnectThread(ConnectThread):
         self.is_connecting = True
 
         # Run pre-connection script.
-        self.run_global_scripts_if_needed(wpath.preconnectscripts)
-        self.run_script_if_needed(self.before_script, 'pre-connection', 'wired', 
+        self.run_global_scripts_if_needed(wpath.preconnectscripts,
+                                          extra_parameters=('wired', 'wired',
+                                                    self.network['profilename'])
+                                          )
+        self.run_script_if_needed(self.before_script, 'pre-connection', 'wired',
                                   'wired')
 
         # Take down interface and clean up previous connections.
         self.put_iface_down(liface)
         self.release_dhcp_clients(liface)
         self.reset_ip_addresses(liface)
+        self.stop_wpa(liface)
         self.flush_routes(liface)
+        self.flush_dns_addresses(liface)
         
         # Bring up interface.
         self.put_iface_up(liface)
         
+        # Manage encryption.
+        if self.network.get('encryption_enabled'):
+            liface.Authenticate(self.network)
+        
         # Set gateway, IP adresses, and DNS servers.
         self.set_broadcast_address(liface)
         self.set_ip_address(liface)
         self.set_dns_addresses(liface)
         
         # Run post-connection script.
-        self.run_global_scripts_if_needed(wpath.postconnectscripts)
-        self.run_script_if_needed(self.after_script, 'post-connection', 'wired', 
+        self.run_global_scripts_if_needed(wpath.postconnectscripts,
+                                          extra_parameters=('wired', 'wired',
+                                                self.network['profilename'])
+                                         )
+        self.run_script_if_needed(self.after_script, 'post-connection', 'wired',
                                   'wired')
 
         self.SetStatus('done')
@@ -1076,5 +1245,5 @@ class WiredConnectThread(ConnectThread):
         if self.debug:
             print "IP Address is: " + str(liface.GetIP())
         
-        self.connect_result = "Success"
+        self.connect_result = "success"
         self.is_connecting = False
