@@ -36,6 +36,9 @@ import re
 import random
 import time
 from string import maketrans, translate
+import dbus
+import socket, fcntl
+import shutil
 
 import wpath
 import misc
@@ -43,13 +46,14 @@ from misc import find_path
 
 # Regular expressions.
 _re_mode = (re.I | re.M | re.S)
-essid_pattern = re.compile('.*ESSID:"?(.*?)"?\s*\n', _re_mode)
+essid_pattern = re.compile('.*ESSID:"?(.*?)".*\n', _re_mode)
 ap_mac_pattern = re.compile('.*Address: (.*?)\n', _re_mode)
 channel_pattern = re.compile('.*Channel:?=? ?(\d+)', _re_mode)
 strength_pattern = re.compile('.*Quality:?=? ?(\d+)\s*/?\s*(\d*)', _re_mode)
-altstrength_pattern = re.compile('.*Signal level:?=? ?(\d+)\s*/?\s*(\d*)', _re_mode)
+altstrength_pattern = re.compile('.*Signal level:?=? ?(\d+)\s*/?\s*(\d*)',
+    _re_mode)
 signaldbm_pattern = re.compile('.*Signal level:?=? ?(-\d\d*)', _re_mode)
-bitrates_pattern = re.compile('(\d+\s+\S+/s)', _re_mode)
+bitrates_pattern = re.compile('([\d\.]+)\s+\S+/s', _re_mode)
 mode_pattern = re.compile('.*Mode:([A-Za-z-]*?)\n', _re_mode)
 freq_pattern = re.compile('.*Frequency:(.*?)\n', _re_mode)
 wep_pattern = re.compile('.*Encryption key:(.*?)\n', _re_mode)
@@ -59,29 +63,35 @@ wpa2_pattern = re.compile('(WPA2)', _re_mode)
 
 #iwconfig-only regular expressions.
 ip_up = re.compile(r'flags=[0.9]*<([^>]*)>', re.S)
-ip_pattern = re.compile(r'inet [Aa]d?dr[^.]*:([^.]*\.[^.]*\.[^.]*\.[0-9]*)', re.S)
+ip_pattern = re.compile(r'inet [Aa]d?dr[^.]*:([^.]*\.[^.]*\.[^.]*\.[0-9]*)',
+    re.S)
 ip_pattern1 = re.compile(r'inet ([^.]*\.[^.]*\.[^.]*\.[0-9]*)', re.S)
-bssid_pattern = re.compile('.*[(Access Point)|(Cell)]: (([0-9A-Z]{2}:){5}[0-9A-Z]{2})', _re_mode)
+bssid_pattern = re.compile('.*[(Access Point)|(Cell)]: ' + \
+    '(([0-9A-Z]{2}:){5}[0-9A-Z]{2})', _re_mode)
 bitrate_pattern = re.compile('.*Bit Rate[=:](.*?/s)', _re_mode)
 opmode_pattern = re.compile('.*Mode:(.*?) ', _re_mode)
-authmethods_pattern = re.compile('.*Authentication capabilities :\n(.*?)Current', _re_mode)
+authmethods_pattern = re.compile('.*Authentication capabilities ' + \
+    ':\n(.*?)Current', _re_mode)
 
 # Regular expressions for wpa_cli output
 auth_pattern = re.compile('.*wpa_state=(.*?)\n', _re_mode)
 
 RALINK_DRIVER = 'ralink legacy'
+NONE_DRIVER = 'none'
 
 blacklist_strict = '!"#$%&\'()*+,./:;<=>?@[\\]^`{|}~ '
 blacklist_norm = ";`$!*|><&\\"
 blank_trans = maketrans("", "")
 
 def _sanitize_string(string):
+    """ Sanitize string. """
     if string:
         return translate(str(string), blank_trans, blacklist_norm)
     else:
         return string
 
 def _sanitize_string_strict(string):
+    """ Sanitize string in a stricter way. """
     if string:
         return translate(str(string), blank_trans, blacklist_strict)
     else:
@@ -130,6 +140,31 @@ def GetDefaultGateway():
         print 'couldn\'t retrieve default gateway from route -n'
     return gateway
 
+def isWireless(devname):
+    """
+    Classic-style wifi device classification using linux/wireless.h 
+    This should be extended or replaced by nl80211.h style classification
+    in the future, if wireless.h is fully replaced.
+    """
+    we = None
+    for t in [socket.AF_INET, socket.AF_INET6, socket.AF_IPX, socket.AF_AX25,
+      socket.AF_APPLETALK]:
+        try:
+            sk = socket.socket(t, socket.SOCK_DGRAM)
+        except socket.error:
+            sk = None
+        if sk is None:
+            continue
+        skfd = sk.fileno()
+        try:
+            #define SIOCGIWNAME 0x8b01 in linux/wireless.h
+            # "used to verify the presence of wireless extensions"
+            we = fcntl.ioctl(skfd, 0x8b01, devname)
+        except IOError:
+            pass
+        sk.close()
+    return we is not None
+
 def GetWirelessInterfaces():
     """ Get available wireless interfaces.
 
@@ -140,10 +175,7 @@ def GetWirelessInterfaces():
 
     """
     dev_dir = '/sys/class/net/'
-    ifnames = [iface for iface in os.listdir(dev_dir)
-               if os.path.isdir(dev_dir + iface) and 
-                  'wireless' in os.listdir(dev_dir + iface)]
-    
+    ifnames = [iface for iface in os.listdir(dev_dir) if isWireless(str(iface))]
     return ifnames
 
 def GetWiredInterfaces():
@@ -163,7 +195,7 @@ def GetWpaSupplicantDrivers():
     output = misc.Run(["wpa_supplicant", "-h"])
     try:
         output = output.split("drivers:")[1].split("options:")[0].strip()
-    except:
+    except KeyError:
         print "Warning: Couldn't get list of valid wpa_supplicant drivers"
         return [""]
     patt = re.compile("(\S+)\s+=.*")
@@ -215,7 +247,24 @@ class BaseInterface(object):
         self.flush_tool = None
         self.link_detect = None       
         self.dhcp_object = None
-    
+
+        self.ethtool_cmd = None
+        self.miitool_cmd = None
+        self.ip_cmd = None
+        self.route_cmd = None
+        self.wpa_cli_cmd = None
+        self.resolvconf_cmd = None
+
+        self.dhclient_cmd = None
+        self.dhclient_needs_verbose = False
+        self.udhcpc_cmd = None
+        self.dhcpcd_cmd = None
+        self.pump_cmd = None
+
+        self.kdesu_cmd = None
+        self.gksudo_cmd = None
+        self.ktsuss_cmd = None
+
     def SetDebugMode(self, value):
         """ If True, verbose output is enabled. """
         self.verbose = value
@@ -285,21 +334,25 @@ class BaseInterface(object):
         client_dict = {
             "dhclient" : 
                 {'connect' : r"%(cmd)s -cf %(dhclientconf)s %(iface)s",
+                 'connect_with_hostname' : r"%(cmd)s -cf %(dhclientconf)s %(iface)s",
                  'release' : r"%(cmd)s -r %(iface)s",
                  'id' : misc.DHCLIENT, 
                  },
             "pump" : 
-                { 'connect' : r"%(cmd)s -i %(iface)s -h %(hostname)s",
+                { 'connect' : r"%(cmd)s -i %(iface)s",
+                  'connect_with_hostname' : r"%(cmd)s -i %(iface)s -h %(hostname)s",
                   'release' : r"%(cmd)s -r -i %(iface)s",
                   'id' : misc.PUMP,
                 },
             "dhcpcd" : 
-                {'connect' : r"%(cmd)s -h %(hostname)s --noipv4ll %(iface)s ",
+                {'connect' : r"%(cmd)s --noipv4ll %(iface)s",
+                 'connect_with_hostname' : r"%(cmd)s -h %(hostname)s --noipv4ll %(iface)s ",
                  'release' : r"%(cmd)s -k %(iface)s",
                  'id' : misc.DHCPCD,
                 },
             "udhcpc":
-                {'connect' : r"%(cmd)s -n -i %(iface)s -H %(hostname)s ",
+                {'connect' : r"%(cmd)s -n -i %(iface)s",
+                 'connect_with_hostname' : r"%(cmd)s -n -i %(iface)s -H %(hostname)s ",
                  'release' : r"killall -SIGUSR2 %(cmd)s",
                  'id' : misc.UDHCPC,
                 },
@@ -309,24 +362,25 @@ class BaseInterface(object):
         # cause dhclient doesn't have a handy dandy argument
         # for specifing the hostname to be sent
         if client_name == "dhclient" and flavor:
-            if hostname == None:
-                # <hostname> will use the system hostname
-                # we'll use that if there is hostname passed
-                # that shouldn't happen, though
-                hostname = '<hostname>'
-            print 'attempting to set hostname with dhclient'
-            print 'using dhcpcd or another supported client may work better'
-            dhclient_template = \
-                open(os.path.join(wpath.etc, 'dhclient.conf.template'), 'r')
+            if hostname:
+                print 'attempting to set hostname with dhclient'
+                print 'using dhcpcd or another supported client may work better'
 
-            output_conf = open(dhclient_conf_path, 'w')
+                dhclient_template = \
+                    open(os.path.join(wpath.etc, 'dhclient.conf.template'), 'r')
+                output_conf = open(dhclient_conf_path, 'w')
 
-            for line in dhclient_template.readlines():
-                line = line.replace('$_HOSTNAME', hostname)
-                output_conf.write(line)
+                for line in dhclient_template.readlines():
+                    line = line.replace('# <WICDHOSTNAME>', 'send host-name "%s";' \
+                        % hostname)
+                    output_conf.write(line)
+
+                output_conf.close()
+                dhclient_template.close()
+            else:
+                shutil.copy(os.path.join(wpath.etc, 'dhclient.conf.template'), \
+                    dhclient_conf_path)
 
-            output_conf.close()
-            dhclient_template.close()
             os.chmod(dhclient_conf_path, 0644)
 
         if not client_name or not cmd:
@@ -334,15 +388,20 @@ class BaseInterface(object):
             return ""
             
         if flavor == "connect":
-            if not hostname:
-                hostname = os.uname()[1]
-            return client_dict[client_name]['connect'] % \
+            if hostname:
+                return client_dict[client_name]['connect_with_hostname'] % \
                     { "cmd" : cmd,
                       "iface" : self.iface,
                       "hostname" : hostname,
                       'dhclientconf' : dhclient_conf_path }
+            else:
+                return client_dict[client_name]['connect'] % \
+                    { "cmd" : cmd,
+                      "iface" : self.iface,
+                      'dhclientconf' : dhclient_conf_path }
         elif flavor == "release":
-            return client_dict[client_name]['release'] % {"cmd":cmd, "iface":self.iface}
+            return client_dict[client_name]['release'] % \
+                {"cmd": cmd, "iface": self.iface}
         else:
             return client_dict[client_name]['id']
     
@@ -408,6 +467,7 @@ class BaseInterface(object):
         self.route_cmd = self._find_program_path("route")
             
     def CheckSudoApplications(self):
+        """ Check for available root-gaining 'sudo' applications. """
         self.gksudo_cmd = self._find_program_path("gksudo")
         self.kdesu_cmd = self._find_program_path("kdesu")
         self.ktsuss_cmd = self._find_program_path("ktsuss")
@@ -421,7 +481,8 @@ class BaseInterface(object):
         
         """
         cmd = 'ifconfig ' + self.iface + ' up'
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
         return True
 
@@ -434,7 +495,8 @@ class BaseInterface(object):
         
         """
         cmd = 'ifconfig ' + self.iface + ' down'
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
         return True
     
@@ -443,7 +505,8 @@ class BaseInterface(object):
     def GetIfconfig(self):
         """ Runs ifconfig and returns the output. """
         cmd = "ifconfig %s" % self.iface
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         return misc.Run(cmd)
 
     @neediface("")
@@ -470,7 +533,8 @@ class BaseInterface(object):
             cmd = ''.join([cmd, 'netmask ', netmask, ' '])
         if broadcast:
             cmd = ''.join([cmd, 'broadcast ', broadcast, ' '])
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
     def _parse_dhclient(self, pipe):
@@ -603,7 +667,8 @@ class BaseInterface(object):
         
         """
         cmd = self._get_dhcp_command('connect', hostname)
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         self.dhcp_object = misc.Run(cmd, include_stderr=True, return_obj=True)
         pipe = self.dhcp_object.stdout
         client_dict = { misc.DHCLIENT : self._parse_dhclient,
@@ -625,7 +690,8 @@ class BaseInterface(object):
     def ReleaseDHCP(self):
         """ Release the DHCP lease for this interface. """
         cmd = self._get_dhcp_command("release")
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
     @neediface(False)
@@ -638,9 +704,24 @@ class BaseInterface(object):
         else:
             print "No route manipulation command available!"
             return 
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
+    @neediface(False)
+    def FlushDNS(self):
+        """ Remove added DNS servers from system resolv.conf.
+
+        Only useful for resolvconf-based setups.
+
+        """
+        if self.resolvconf_cmd:
+            cmd = [self.resolvconf_cmd, '-d', self.iface + '.wicd']
+            if self.verbose:
+                print cmd
+            p = misc.Run(cmd, include_stderr=True, return_obj=True)
+            p.communicate()
+
     @neediface(False)
     def SetDNS(self, dns1=None, dns2=None, dns3=None, 
                dns_dom=None, search_dom=None):
@@ -676,8 +757,9 @@ class BaseInterface(object):
             resolv_params += ''.join(valid_dns_list)
 
         if self.resolvconf_cmd:
-            cmd = [self.resolvconf_cmd, '-a', self.iface]
-            if self.verbose: print cmd
+            cmd = [self.resolvconf_cmd, '-a', self.iface + '.wicd']
+            if self.verbose:
+                print cmd
             p = misc.Run(cmd, include_stderr=True, return_obj=True)
             p.communicate(input=resolv_params)
         else:
@@ -696,7 +778,8 @@ class BaseInterface(object):
             print "No flush command available!"
             cmds = []
         for cmd in cmds:
-            if self.verbose: print cmd
+            if self.verbose:
+                print cmd
             misc.Run(cmd)
 
     @neediface(False)
@@ -711,7 +794,8 @@ class BaseInterface(object):
             print 'WARNING: Invalid gateway found.  Aborting!'
             return False
         cmd = 'route add default gw %s dev %s' % (gw, self.iface)
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
     @neediface("")
@@ -729,7 +813,8 @@ class BaseInterface(object):
         # check multiple ifconfig output styles
         for pat in [ip_pattern, ip_pattern1]:
             m = misc.RunRegex(pat, output)
-            if m: return m
+            if m:
+                return m
         return None
 
     @neediface(False)
@@ -750,7 +835,8 @@ class BaseInterface(object):
             # timeout, while the above will wait (-w) 3 seconds at
             # most.
             cmd = "ping -q -c 1 %s" % gateway
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         return misc.LaunchAndWait(cmd)
 
     @neediface(False)
@@ -765,7 +851,8 @@ class BaseInterface(object):
         try:
             flags = open(flags_file, "r").read().strip()
         except IOError:
-            print "Could not open %s, using ifconfig to determine status" % flags_file
+            print "Could not open %s, using ifconfig to determine status" \
+                % flags_file
             return self._slow_is_up(ifconfig)
         return bool(int(flags, 16) & 1)
     
@@ -773,7 +860,8 @@ class BaseInterface(object):
     def StopWPA(self):
         """ Terminates wpa using wpa_cli"""
         cmd = 'wpa_cli -i %s terminate' % self.iface
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
         
         
@@ -832,7 +920,8 @@ class BaseWiredInterface(BaseInterface):
             while True:
                 tries += 1
                 time.sleep(2)
-                if self.IsUp() or tries > MAX_TRIES: break
+                if self.IsUp() or tries > MAX_TRIES:
+                    break
       
         if os.path.exists(carrier_path):
             carrier = open(carrier_path, 'r')
@@ -844,7 +933,8 @@ class BaseWiredInterface(BaseInterface):
                 elif link == 0:
                     return False
             except (IOError, ValueError, TypeError):
-                print 'Error checking link using /sys/class/net/%s/carrier' % self.iface
+                print 'Error checking link using /sys/class/net/%s/carrier' \
+                    % self.iface
                 
         if self.ethtool_cmd and self.link_detect in [misc.ETHTOOL, misc.AUTO]:
             return self._eth_get_plugged_in()
@@ -867,10 +957,13 @@ class BaseWiredInterface(BaseInterface):
             print 'Wired Interface is down, putting it up'
             self.Up()
             time.sleep(6)
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         tool_data = misc.Run(cmd, include_stderr=True)
-        if misc.RunRegex(re.compile('(Link detected: yes)', re.I | re.M  | re.S),
-                         tool_data):
+        if misc.RunRegex(
+          re.compile('(Link detected: yes)', re.I | re.M  | re.S),
+          tool_data
+        ):
             return True
         else:
             return False
@@ -883,14 +976,16 @@ class BaseWiredInterface(BaseInterface):
         
         """
         cmd = "%s %s" % (self.miitool_cmd, self.iface)
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         tool_data = misc.Run(cmd, include_stderr=True)
         if misc.RunRegex(re.compile('(Invalid argument)', re.I | re.M  | re.S), 
                          tool_data) is not None:
             print 'Wired Interface is down, putting it up'
             self.Up()
             time.sleep(4)
-            if self.verbose: print cmd
+            if self.verbose:
+                print cmd
             tool_data = misc.Run(cmd, include_stderr=True)
         
         if misc.RunRegex(re.compile('(link ok)', re.I | re.M | re.S),
@@ -900,11 +995,13 @@ class BaseWiredInterface(BaseInterface):
             return False
         
     def Authenticate(self, network):
+        """ Authenticate with wpa_supplicant. """
         misc.ParseEncryption(network)
         cmd = ['wpa_supplicant', '-B', '-i', self.iface, '-c',
                os.path.join(wpath.networks, 'wired'),
                '-Dwired']
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
 class BaseWirelessInterface(BaseInterface):
@@ -934,7 +1031,8 @@ class BaseWirelessInterface(BaseInterface):
 
         """
         cmd = ['iwconfig', self.iface, 'essid', '--', str(essid)]
-        if self.verbose: print str(cmd)
+        if self.verbose:
+            print str(cmd)
         misc.Run(cmd)
 
     @neediface(False)
@@ -960,7 +1058,8 @@ class BaseWirelessInterface(BaseInterface):
     def GetIwconfig(self):
         """ Returns the output of iwconfig for this interface. """
         cmd = "iwconfig " + self.iface
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         return misc.Run(cmd)
 
     def _FreqToChannel(self, freq):
@@ -985,7 +1084,7 @@ class BaseWirelessInterface(BaseInterface):
         try:
             ret = freq_dict[freq]
         except KeyError:
-            print "Couldn't determine channel number for frequency: " + str(freq)
+            print "Couldn't determine channel number for frequency:", str(freq)
         
         return ret
 
@@ -1006,7 +1105,7 @@ class BaseWirelessInterface(BaseInterface):
         for x in lines:
             ap = {}
             info = x.split("   ")
-            info = filter(None, [x.strip() for x in info])
+            info = [x.strip() for x in info if x.strip()]
             if len(info) < 5:
                 continue
             if re.match(patt, info[2].upper()):
@@ -1032,7 +1131,7 @@ class BaseWirelessInterface(BaseInterface):
                 ap['enctype'] = info[4 + offset]
             elif info[5 + offset] == 'WPA2-PSK':
                 ap['encryption_method'] = 'WPA2'
-                ap['authmode'] ="WPA2PSK"
+                ap['authmode'] = "WPA2PSK"
                 ap['keyname'] = "WPA2PSK"
                 ap['enctype'] = info[4 + offset]
             elif info[4 + offset] == "NONE":
@@ -1041,7 +1140,8 @@ class BaseWirelessInterface(BaseInterface):
                 print "Unknown AuthMode, can't assign encryption_method!"
                 ap['encryption_method'] = 'Unknown'
             aps[bssid] = ap
-        if self.verbose: print str(aps)
+        if self.verbose:
+            print str(aps)
         return aps
 
     def _ParseRalinkAccessPoint(self, ap, ralink_info, cell):
@@ -1056,7 +1156,6 @@ class BaseWirelessInterface(BaseInterface):
         Updated array containing info about the current access point
 
         """
-        wep_pattern = re.compile('.*Encryption key:(.*?)\n', re.I | re.M  | re.S)
         if ralink_info.has_key(ap['bssid']):
             info = ralink_info[ap['bssid']]
             for key in info.keys():
@@ -1076,10 +1175,13 @@ class BaseWirelessInterface(BaseInterface):
 
         """
         mode = _sanitize_string_strict(mode)
+        if not mode:
+            mode = 'master'
         if mode.lower() == 'master':
             mode = 'managed'
         cmd = 'iwconfig %s mode %s' % (self.iface, mode)
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
     @neediface(False)
@@ -1095,7 +1197,8 @@ class BaseWirelessInterface(BaseInterface):
             return False
         
         cmd = 'iwconfig %s channel %s' % (self.iface, str(channel))
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         misc.Run(cmd)
 
     @neediface(False)
@@ -1107,7 +1210,27 @@ class BaseWirelessInterface(BaseInterface):
 
         """
         cmd = 'iwconfig %s key %s' % (self.iface, key)
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
+        misc.Run(cmd)
+
+    @neediface(False)
+    def SetBitrate(self, bitrate, allow_lower=False):
+        ''' Set the desired bitrate for the interface.
+
+        Keyword arguments:
+        bitrate -- desired bitrate (string)
+        allow_lower -- whether to allow lower bitrates, or keep it fixed (bool)
+        '''
+        # FIXME: what if, in future, bitrates won't be "M(egabit per second)"
+        #+anymore?
+        if bitrate == 'auto':
+            cmd = 'iwconfig %s rate auto'
+        else:
+            if allow_lower:
+                cmd = 'iwconfig %s rate %sM auto' % (self.iface, bitrate)
+            else:
+                cmd = 'iwconfig %s rate %sM fixed' % (self.iface, bitrate)
         misc.Run(cmd)
 
     @neediface(False)
@@ -1124,11 +1247,13 @@ class BaseWirelessInterface(BaseInterface):
         base = "iwconfig %s" % self.iface
         if channel and str(channel).isdigit():
             cmd = "%s channel %s" % (base, str(channel))
-            if self.verbose: print cmd
+            if self.verbose:
+                print cmd
             misc.Run(cmd)
         if bssid:
             cmd = "%s ap %s" % (base, bssid)
-            if self.verbose: print cmd
+            if self.verbose:
+                print cmd
             misc.Run(cmd)
         
     def GeneratePSK(self, network):
@@ -1139,11 +1264,13 @@ class BaseWirelessInterface(BaseInterface):
         
         """
         wpa_pass_path = misc.find_path('wpa_passphrase')
-        if not wpa_pass_path: return None
+        if not wpa_pass_path:
+            return None
         key_pattern = re.compile('network={.*?\spsk=(.*?)\n}.*',
                                  re.I | re.M  | re.S)
         cmd = [wpa_pass_path, str(network['essid']), str(network['key'])]
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         return misc.RunRegex(key_pattern, misc.Run(cmd))
 
     @neediface(False)
@@ -1158,11 +1285,16 @@ class BaseWirelessInterface(BaseInterface):
         if self.wpa_driver == RALINK_DRIVER:
             self._AuthenticateRalinkLegacy(network)
         else:
+            if self.wpa_driver == NONE_DRIVER:
+                driver = ''
+            else:
+                driver = '-D' + self.wpa_driver
             cmd = ['wpa_supplicant', '-B', '-i', self.iface, '-c',
                    os.path.join(wpath.networks, 
                                 network['bssid'].replace(':', '').lower()),
-                   '-D', self.wpa_driver]
-            if self.verbose: print cmd
+                   driver]
+            if self.verbose:
+                print cmd
             misc.Run(cmd)
 
     def _AuthenticateRalinkLegacy(self, network):
@@ -1187,7 +1319,8 @@ class BaseWirelessInterface(BaseInterface):
                 print 'Setting up WEP'
                 cmd = ''.join(['iwconfig ', self.iface, ' key ',
                               network.get('key')])
-                if self.verbose: print cmd
+                if self.verbose:
+                    print cmd
                 misc.Run(cmd)
             else:
                 cmd_list = []
@@ -1195,13 +1328,14 @@ class BaseWirelessInterface(BaseInterface):
                 cmd_list.append('AuthMode=' + info['authmode'])
                 cmd_list.append('EncrypType=' + info['enctype'])
                 cmd_list.append('SSID="%s"' % network['essid'])
-                cmd_list.append('%s="%s"' % (network['keyname'], network['key']))
+                cmd_list.append('%(keyname)s="%(key)s"' % network)
                 if info['nettype'] == 'SHARED' and info['enctype'] == 'WEP':
                     cmd_list.append('DefaultKeyID=1')
     
                 for cmd in cmd_list:
                     cmd = ['iwpriv', self.iface, 'set', cmd]
-                    if self.verbose: print ' '.join(cmd)
+                    if self.verbose:
+                        print ' '.join(cmd)
                     misc.Run(cmd)
 
     @neediface([])
@@ -1213,7 +1347,8 @@ class BaseWirelessInterface(BaseInterface):
 
         """
         cmd = 'iwlist ' + self.iface + ' scan'
-        if self.verbose: print cmd
+        if self.verbose:
+            print cmd
         results = misc.Run(cmd)
         # Split the networks apart, using Cell as our split point
         # this way we can look at only one network at a time.
@@ -1286,9 +1421,14 @@ class BaseWirelessInterface(BaseInterface):
             ap['channel'] = self._FreqToChannel(freq)
 
         # Bit Rate
-        ap['bitrates'] = misc.RunRegex(bitrates_pattern,
-                                       cell.split("Bit Rates")[-1])
-     
+        bitrates = cell.split('Bit Rates')[-1].replace('\n', '; ')
+        m = re.findall(bitrates_pattern, bitrates)
+        if m:
+            # numeric sort
+            ap['bitrates'] = sorted(m, lambda x, y: int(float(x) - float(y)))
+        else:
+            ap['bitrates'] = None
+
         # BSSID
         ap['bssid'] = misc.RunRegex(ap_mac_pattern, cell)
 
@@ -1440,7 +1580,8 @@ class BaseWirelessInterface(BaseInterface):
         """ Get the available authentication methods for the interface. """
         if not iwlistauth:
             cmd = 'iwlist ' + self.iface + ' auth'
-            if self.verbose: print cmd
+            if self.verbose:
+                print cmd
             output = misc.Run(cmd)
         else:
             output = iwlistauth
@@ -1449,6 +1590,21 @@ class BaseWirelessInterface(BaseInterface):
         authm_list = [m.strip() for m in authm.split('\n') if m.strip()]
         return ';'.join(authm_list)
 
+    @neediface('')
+    def GetAvailableBitrates(self):
+        """ Get the available bitrates the wifi card can use. """
+
+        cmd = 'iwlist ' + self.iface + ' rate'
+        if self.verbose:
+            print cmd
+        rates = misc.Run(cmd)
+
+        # process the output
+        rates = rates.split('\n')
+        rates = [x.strip().split(' ')[0] for x in rates]
+        rates = [x for x in rates if x and x[0].isdigit()]
+        return dbus.Array(rates, signature='v')
+
     def _get_link_quality(self, output):
         """ Parse out the link quality from iwlist scan or iwconfig output. """
         try:
